#
# "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer(a) for its own 
# instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
# or for redistribution by customer, as part of such an application, in customer's own products."
#
# 5655-W05, 5655-W09 (C) COPYRIGHT International Business Machines Corp., 2008, 2011
# All Rights Reserved * Licensed Materials - Property of IBM
#
# ConsolidateJAASAuthAliases
#
# BPM and WESB for z/OS V7.5 JAAS Authentication Alias consolidation script.
# 
# Replace several auth aliases created by the WPS or WESB augment process with a single entry.
#
# The script will delete the following auth aliases, replace them with a single auth alias
#   built from the script arguments, and reassign the new auth alias to resources which
#   previously used the old ones:
#   BPCDB_<qualifier>_Auth_Alias
#   BPCME_00_Auth_Alias
#   BSPACE_Auth_Alias 
#   CEIME_<qualifier>_Auth_Alias
#   OBSVRDB_<qualifier>_Auth_Alias
#   PERFDWME_Auth_Alias
#   PROCSVRME_Auth_Alias
#   SCAAPPME00_Auth_Alias
#   SCASYSME00_Auth_Alias
#   WPSDB_Auth_Alias
#   performancedblogon
#   processdblogon
#
# Usage: wsadmin
#          -host <HostName>
#          -port <HostPort>
#          -lang jython
#          -f ConsolidateJAASAuthAliases.py
#          <JAAS Authentication Alias Name>
#          <User ID>
#          <Password>
#          <Scan Mode> (Optional)
#
#   <HostName>                       : Host name of the application server (Standalone) or Deployment Manager (ND).
#   <HostPort>                       : SOAP port of the application server (Standalone) or Deployment Manager (ND).
#   <JAAS Authentication Alias Name> : Name of the new JAAS authentication alias.
#   <User ID>                        : User identifier of the new JAAS authentication alias.
#   <Password>                       : Password of the new JAAS authentication alias.
#   <Scan Mode> (Optional)           : Any character string will "dry run" (updates not committed) the script.
#

import sys

#
# Validate the arguments
#
def validateArgs(args):

   global newAliasName
   global newAliasUser
   global newAliasPassword
   global newAliasDescription
   global scanMode

# By default run in scan mode (do not commit any updates)
   scanMode = 1

   if (len(args) != 3) and (len(args) != 4):
      printSyntax()
      return 1
   else:
      newAliasName = args[0]
      newAliasUser = args[1]
      newAliasPassword = args[2]
      newAliasDescription = "Common WPS database access (Created by ConsolidateJAASAuthAliases.py)"

   if len(args) != 4:
# Run for real
      scanMode = 0

#
# Remove duplicate aliases and replace them with a new one
#
def processAliases():

# Create the new alias
   newAlias = createNewAlias()

# Retrieve list of aliases to replace 
   doomedAliases = getDoomedAliases()

# Modify resources with the new alias and remove the old one
   for alias in doomedAliases:
      replaceAliasReferences(alias, newAlias)
      print "\n   Removing alias: " + AdminConfig.showAttribute(alias, "alias")
      AdminConfig.remove(alias)

#
# Create the new JAAS authentication alias from the input arguments
#
def createNewAlias():

   securityId = AdminConfig.list("Security")

   aliasAttr = ["alias", newAliasName]
   userAttr = ["userId", newAliasUser]
   passwordAttr = ["password", newAliasPassword]
   descrAttr = ["description", newAliasDescription]
   attrList = [aliasAttr, userAttr, passwordAttr, descrAttr]

   newJAASAlias = AdminConfig.create("JAASAuthData", securityId, attrList)
   print "\n   Created JAAS alias: " + AdminConfig.showAttribute(newJAASAlias, "alias")

   return newJAASAlias

#
# Get aliases to replace
#
def getDoomedAliases():

   listOfAliasIdentifiers = ["BPCDB_", "BPCME_", "BSPACE_Auth_Alias", "CEIME_", "OBSVRDB_", "PERFDWME_", "PROCSVRME_", "SCAAPPME", "SCASYSME", "WPSDB_Auth_Alias", "performancedblogon", "processdblogon"] 

   listOfAllAliases = AdminConfig.list("JAASAuthData").split("\n")
   listOfDoomedAliases = []

   for alias in listOfAllAliases:
      aliasId = AdminConfig.showAttribute(alias, "alias")
      for identifier in listOfAliasIdentifiers:
         if aliasId == identifier:
            listOfDoomedAliases.append(alias)
         elif aliasId.startswith(identifier):
            listOfDoomedAliases.append(alias)
         elif aliasId.endswith(identifier):
            listOfDoomedAliases.append(alias)

   return listOfDoomedAliases

#
# Replace alias references
#
def replaceAliasReferences(doomedAlias, newAlias):

   doomedAliasName = AdminConfig.showAttribute(doomedAlias, "alias")

# Modify data sources
   listOfDataSources = AdminConfig.list("DataSource").split("\n")
   for dataSource in listOfDataSources:
      if doomedAliasName == AdminConfig.showAttribute(dataSource, "authDataAlias"):
         print "\n   Replacing alias reference in data source: " + AdminConfig.showAttribute(dataSource, "name")
         print "      " + doomedAliasName + " => " + newAliasName
         AdminConfig.modify(dataSource, [["authDataAlias", newAliasName]])
      if doomedAliasName == AdminConfig.showAttribute(dataSource, "xaRecoveryAuthAlias"):
         print "\n   Replacing XA recovery alias reference in data source: " + AdminConfig.showAttribute(dataSource, "name")
         print "      " + doomedAliasName + " => " + newAliasName
         AdminConfig.modify(dataSource, [["xaRecoveryAuthAlias", newAliasName]])

# Modify bus messaging engine data stores
   listOfBusMEs = AdminConfig.list("SIBMessagingEngine").split("\n")
   for busME in listOfBusMEs:
      busDataStore = AdminConfig.showAttribute(busME, "dataStore")
      if doomedAliasName == AdminConfig.showAttribute(busDataStore, "authAlias"):
         print "\n   Replacing alias reference in SIBus data store of ME: " + AdminConfig.showAttribute(busME, "name")
         print "      " + doomedAliasName + " => " + newAliasName
         AdminConfig.modify(busDataStore, [["authAlias", newAliasName]])

# Modify CMP connection factories
   listOfCMPConnectorFactories = AdminConfig.list("CMPConnectorFactory").split("\n")
   for CMPConnectorFactory in listOfCMPConnectorFactories:
      CMPCFAuthAlias = AdminConfig.showAttribute(CMPConnectorFactory, "authDataAlias")
      if doomedAliasName == CMPCFAuthAlias:
         print "\n   Replacing alias reference in CMP connection factory: " + AdminConfig.showAttribute(CMPConnectorFactory, "name")
         print "      Component-managed " + doomedAliasName
         AdminConfig.modify(CMPConnectorFactory, [["authDataAlias", newAliasName]])
      CMPCFXaRecoveryAlias = AdminConfig.showAttribute(CMPConnectorFactory, "xaRecoveryAuthAlias")
      if doomedAliasName == CMPCFXaRecoveryAlias:
         print "\n   Replacing XA recovery alias reference in CMP connection factory: " + AdminConfig.showAttribute(CMPConnectorFactory, "name")
         print "      " + doomedAliasName + " => " + newAliasName
         AdminConfig.modify(CMPConnectorFactory, [["xaRecoveryAuthAlias", newAliasName]])

#
# Print syntax
#
def printSyntax():

   print "\n   Syntax: wsadmin -lang jython -f ConsolidateJAASAuthAliases.py <Other WSADMIN Parameters> <JAAS Authentication Alias Name> <User ID> <Password> <Scan Mode>"
   print "      <Other WSADMIN Parameters>: WSADMIN runtime parameters such as server host name and port number."
   print "      <JAAS Authentication Alias Name>: Name of the new JAAS authentication alias."
   print "      <User ID>: User identifier of the new JAAS authentication alias."
   print "      <Password>: Password of the new JAAS authentication alias."
   print "      <Scan Mode> (Optional): Any character string will \"dry run\" (updates not committed) the script."

#
# Main
#
print "\nConsolidateJAASAuthAliases: Starting"

#
# Validate arguments
#
if validateArgs(sys.argv) != 1:
   processAliases()

if scanMode == 0:
   print "\n   Saving configuration"
   AdminConfig.save()
else:
   print "\n   Running in scan mode, no updates committed"
   AdminConfig.reset()

print "\nConsolidateJAASAuthAliases: Completed"